@@ -79,7 +79,7 @@ def getInputDataLengthHdf5(filepath, groupname=None):
     return nrow
 
 
-def initializeHdf5Write(filepath, groupname=None, **kwds):
+def initializeHdf5WriteSingle(filepath, groupname=None, **kwds):
     """ Prepares an hdf5 file for output
 
     Parameters
@@ -106,26 +106,72 @@ def initializeHdf5Write(filepath, groupname=None, **kwds):
     dtype : `str`
         The data type for this dataset
 
-    For exmaple
-    `initialize_writeout('test.hdf5', scalar=((100000,), 'f4'), vect=((100000, 3), 'f4'))`
-
+   For exmaple
+    `initializeHdf5WriteSingle('test.hdf5', data = dict(scalar=((100000,), 'f4'), vect=((100000, 3), 'f4'))`
     Would initialize an hdf5 file with two datasets, with shapes and data types as given
+
     """
     outdir = os.path.dirname(os.path.abspath(filepath))
     if not os.path.exists(outdir):  #pragma: no cover
         os.makedirs(outdir, exist_ok=True)
     outf = h5py.File(filepath, "w")
-    if groupname is None:  #pragma: no cover
+    if groupname is None:
         group = outf
     else:
         group = outf.create_group(groupname)
+    for key, shape in kwds.items():
+        group.create_dataset(key, shape[0], shape[1])
+    return group, outf        
 
+
+def initializeHdf5Write(filepath, **kwds):
+    """ Prepares an hdf5 file for output
+
+    Parameters
+    ----------
+    filepath : `str`
+        The output file name
+
+    Returns
+    -------
+    group : `h5py.File` or `h5py.Group`
+        The group to write to
+    fout : `h5py.File`
+        The output file
+
+    Notes
+    -----
+    The keywords should be used to create groups within the hdf5 file.
+    Each keyword should provide a dictionary with the data set information of the form:
+     group = {'data1' : ( (shape1), (dtype1) ), 'data2' : ( (shape2), (dtype2) )}
+
+    group : `str` 
+        Name of the Hdf5 group
+    data  : `str`
+        Name of the column to be written
+    shape : `tuple` ( `int` )
+        The shape of the data for this dataset
+    dtype : `str`
+        The data type for this dataset
+
+    For exmaple
+    `initializeHdf5Write('test.hdf5', data = dict(scalar=((100000,), 'f4'), vect=((100000, 3), 'f4'))`
+
+    Would initialize an hdf5 file with one group and two datasets, with shapes and data types as given
+    """
+    outdir = os.path.dirname(os.path.abspath(filepath))
+    if not os.path.exists(outdir):  #pragma: no cover
+        os.makedirs(outdir, exist_ok=True)
+    outf = h5py.File(filepath, "w")
+    groups = {}
     for k, v in kwds.items():
-        group.create_dataset(k, v[0], v[1])
-    return group, outf
+        group = outf.create_group(k)
+        groups[k] = group
+        for key, shape in v.items():
+            group.create_dataset(key, shape[0], shape[1])
+    return groups, outf
 
-
-def writeDictToHdf5Chunk(fout, odict, start, end, **kwds):
+def writeDictToHdf5ChunkSingle(fout, odict, start, end, **kwds):
     """ Writes a data chunk to an hdf5 file
 
     Parameters
@@ -160,6 +206,43 @@ def writeDictToHdf5Chunk(fout, odict, start, end, **kwds):
         k_out = kwds.get(key, key)
         fout[k_out][start:end] = val
 
+        
+def writeDictToHdf5Chunk(groups, odict, start, end, **kwds):
+    """ Writes a data chunk to an hdf5 file
+
+    Parameters
+    ----------
+    fout : `h5py.File`
+        The file
+
+    odict : `OrderedDict`, (`str`, `numpy.array`)
+        The data being written
+
+    start : `int`
+        Starting row number to place the data
+
+    end : `int`
+        Ending row number to place the data
+
+    Notes
+    -----
+    The kwds can be used to control the output locations, i.e., to
+    rename the columns in data_dict when they good into the output file.
+
+    For each item in data_dict, the output location is set as
+
+    `k_out = kwds.get(key, key)`
+
+    This will check the kwds to see if they contain `key` and if so, will
+    return the corresponding value.  Otherwise it will just return `key`.
+
+    I.e., if `key` is present in kwds in will override the name.
+    """
+    for group_name, group in groups.items():
+        for key, val in odict[group_name].items():
+            k_out = kwds.get(key, key)
+            group[k_out][start:end] = val
+
 
 def finalizeHdf5Write(fout, groupname=None, **kwds):
     """ Write any last data and closes an hdf5 file
@@ -181,8 +264,54 @@ def finalizeHdf5Write(fout, groupname=None, **kwds):
         group[k] = v
     fout.close()
 
+def split_tasks_by_rank(tasks, parallel_size, rank):
+    """Iterate through a list of items, yielding ones this process is responsible for/
 
-def iterHdf5ToDict(filepath, chunk_size=100_000, groupname=None):
+    Tasks are allocated in a round-robin way.
+
+    Parameters
+    ----------
+        tasks: Tasks to split up (iterator)  
+        parallel_size: the number of processes under MPI (int)
+        rank: the rank of this process under MPI (int)
+
+    Returns
+        output: number of the first task for this process (iterator)
+    """
+    for i, task in enumerate(tasks):
+        if i % parallel_size == rank:
+            yield task
+
+def data_ranges_by_rank(n_rows, chunk_rows, parallel_size, rank):
+    """Split a number of rows by process.
+
+    Given a total number of rows to read and a chunk size, yield
+    the ranges within them that this process should handle.
+
+    Parameters
+    ----------
+    n_rows: Total number of rows to split up (int)
+    chunk_rows: Size of each chunk to be read (int)
+    parallel_size: the number of processes under MPI (int)
+    rank: the rank of this process under MPI (int)
+
+    Returns
+        output:
+        iterator chunk
+
+        start: start index (int)
+        end: ending index (int)
+    """
+    n_chunks = n_rows // chunk_rows
+    if n_chunks * chunk_rows < n_rows:  # pragma: no cover
+        n_chunks += 1
+    it = split_tasks_by_rank(range(n_chunks), parallel_size, rank)
+    for i in it:
+        start = i * chunk_rows
+        end = min((i + 1) * chunk_rows, n_rows)
+        yield start, end
+
+def iterHdf5ToDict(filepath, chunk_size=100_000, groupname=None, rank=0, parallel_size=1):
     """
     iterator for sending chunks of data in hdf5.
 
@@ -190,6 +319,8 @@ def iterHdf5ToDict(filepath, chunk_size=100_000, groupname=None):
     ----------
       filepath: input file name (str)
       chunk_size: size of chunk to iterate over (int)
+      rank: the rank of this process under MPI (int)
+      parallel_size: the number of processes under MPI (int)
 
     Returns
     -------
@@ -201,12 +332,13 @@ def iterHdf5ToDict(filepath, chunk_size=100_000, groupname=None):
         end: ending index (int)
         data: dictionary of all data from start:end (dict)
     """
+    if rank>=parallel_size:
+        raise TypeError(f"MPI rank {rank} larger than the total number of processes {parallel_size}") #pragma: no cover
     f, infp = readHdf5Group(filepath, groupname)
     num_rows = getGroupInputDataLength(f)
+    ranges = data_ranges_by_rank(num_rows, chunk_size, parallel_size, rank)
     data = OrderedDict()
-    for i in range(0, num_rows, chunk_size):
-        start = i
-        end = min(i+chunk_size, num_rows)
+    for start, end in ranges:
         for key, val in f.items():
             data[key] = readHdf5DatasetToArray(val, start, end)
         yield start, end, data
